/*** File: Edge.java
*** Contains: Class defining the directed edge element of CPE.
*** Author: Steve Polyak <Steve_Polyak@ed.ac.uk>
*** Created: June 1997
*** Updated: Wed Jan 21 19:21:33 1998 by Jeff Dalton
*/

package ix.acp3;

import java.util.*;
import java.awt.*;
// import Node;

class Edge {
  Node m_from;
  Node m_to;
  final Color m_edgeColor = Color.black;
  boolean m_selected = false;
  final Color m_selectedColor = Color.lightGray;
  
  Edge(Node from, Node to){
    m_from = from;
    m_to = to;
  }
  
  // getRect() appears not to get called anywhere
  public Rectangle getRect(){
    int x,y,width,height;
    
    if (m_from.m_x < m_to.m_x) {
      x = m_from.m_x;
      width = m_to.m_x - m_from.m_x;
    }
    else {
      x = m_to.m_x;
      width = m_from.m_x - m_to.m_x;
    }
    if (m_from.m_y < m_to.m_y) {
      y = m_from.m_y;
      height = m_to.m_y - m_from.m_y;
    }
    else {
      y = m_to.m_y;
      height = m_from.m_y - m_to.m_y;
    }
    
    if (width == 0)
      width = 1;
    if (height == 0)
      height = 1;
    
    return new Rectangle(x,y,width,height);
  }
  
  public void draw(Graphics g, FontMetrics fm) {
    int x1 = m_from.m_x;
    int y1 = m_from.m_y;
    int x2 = m_to.m_x;
    int y2 = m_to.m_y;
    
    if (m_selected)
      g.setColor(m_selectedColor);
    else
      g.setColor(m_edgeColor);
    
    // 1st need to adjust for the size of the "to" node
    int w = fm.stringWidth(m_to.m_key);
    int h = fm.getHeight();
    if (m_to.dummyNode) {
      g.drawLine(x1, y1, x2, y2);
    }
    else {
      g.drawLine(x1, y1, (x2 - w/2), y2);
    }
  }
  
  public void drawArrow(Graphics g, FontMetrics fm) {
    int x1 = m_from.m_x;
    int y1 = m_from.m_y;
    int x2 = m_to.m_x;
    int y2 = m_to.m_y;

    if (m_to.dummyNode) {
      return;
    }
    if (m_selected)
      g.setColor(m_selectedColor);
    else
      g.setColor(m_edgeColor);
    
    // 1st need to adjust for the size of the "to" node
    int w = fm.stringWidth(m_to.m_key);
    int h = fm.getHeight();
    placeArrow(g, x1, y1, (x2 - w/2), y2, 4);
  }
  
  public void placeArrow(Graphics g, int ax,int ay,int bx,int by, int r) {
    
    int dx = bx-ax;
    int dy = by-ay;
    double length = Math.sqrt(dx*dx+dy*dy);
    double sin = dy/length;
    double cos = dx/length;
    double px = length-r;
    double py = 0;
    double lx = px-2*r;
    double ly = r;
    double rx = lx;
    double ry = -ly;
    double mx = px-(3*r/2);
    double my = 0;
    double tx;
    double ty;
    tx = px*cos-py*sin;
    ty = px*sin+py*cos;
    px = tx; py = ty;
    tx = lx*cos-ly*sin;
    ty = lx*sin+ly*cos;
    lx = tx; ly = ty;
    tx = rx*cos-ry*sin;
    ty = rx*sin+ry*cos;
    rx = tx; ry = ty;
    tx = mx*cos-my*sin;
    ty = mx*sin+my*cos;
    mx = tx; my = ty;
    px+=ax; py+=ay;
    mx+=ax; my+=ay;
    lx+=ax; ly+=ay;
    rx+=ax; ry+=ay;
    int[] xvals = new int[4];
    int[] yvals = new int[4];
    xvals[0]=(int)px;
    xvals[1]=(int)lx;
    xvals[2]=(int)mx;
    xvals[3]=(int)rx;
    yvals[0]=(int)py;
    yvals[1]=(int)ly;
    yvals[2]=(int)my;
    yvals[3]=(int)ry;
    g.fillPolygon(xvals,yvals,4);
  }
  
}
