/** File: ViewGraph.java
*** Contains: Visual class that the process elements sit on in CPE.
*** Author: Steve Polyak <Steve_Polyak@ed.ac.uk>
*** Created: June 1997
*** Updated: Sun Aug 23 03:11:41 1998 by Jeff Dalton
*/

package ix.acp3;

import java.util.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.applet.Applet;
import ix.util.*;

class ViewGraph extends Panel {
  
  MainFrame m_frame;
  CPEProcess m_process;
  
  private Dimension myDimension;

  public String currentGraph = "blank";

  public int currentCOA = 0;

  private Vector statusTables = new Vector();  // a vector of Hashtables

  private Hashtable ht0 = new Hashtable();
  private Hashtable ht1 = new Hashtable();
  private Hashtable ht2 = new Hashtable();
  private Hashtable ht3 = new Hashtable();

  ViewGraph(MainFrame frame) {
    m_frame = frame;
    m_process = new CPEProcess(this);
    this.setBackground(Color.white);
    myDimension = new Dimension(706,202);
  }

  public synchronized void update(Graphics g) {
    m_process.update(g);
  }

  public void paint(Graphics g) {
    m_process.update(g);
  }

  //  public synchronized boolean mouseUp(Event evt, int x, int y) {
  //    switchGraphs();
  //    return true;
  //  }

  public synchronized boolean mouseUp(Event evt, int x, int y) {
    return m_process.mouseUp(evt, x, y);
  }

  //  public void switchGraphs() {
  //    if (currentGraph.equals("blank")) {
  //      return;
  //    }
  //    if (currentGraph.equals("top_level.txt")) {
  //      setGraph("dev_COA.txt");
  //      return;
  //    }
  //    if (currentGraph.equals("dev_COA.txt")) {
  //      setGraph("top_level.txt");
  //      return;
  //    }
  //  }

  public Dimension minimumSize() {
    return myDimension;
  }
  
  public Dimension preferredSize() {
    return myDimension;
  }

  public synchronized void start() {
    m_process.newProcess();
    m_process.clear();
    currentGraph = "blank";
    for (Enumeration e = statusTables.elements(); e.hasMoreElements(); ) {
      Hashtable ht = (Hashtable)e.nextElement();
      ht.clear();
    }
    repaint();
  }

  public synchronized void setCurrentCOA(int coa) {
    currentCOA = coa;
    setGraph(currentGraph);
  }

  public synchronized void setGraph(String graph) {
    currentGraph = graph;
    loadBlankGraph();
    Hashtable ht = (Hashtable) statusTables.elementAt(currentCOA);
    for (Enumeration e = ht.keys(); e.hasMoreElements(); ) {
      String node = (String) e.nextElement();
      Integer status = (Integer) ht.get(node);
      setNodeColour(node, status.intValue());
    }
  }

  public void setStatus(int coa, String node, int status) {
    Hashtable ht = (Hashtable) statusTables.elementAt(coa);
    ht.put(node, new Integer(status));
    if (coa == currentCOA) {
      setNodeColour(node, status);
    }
  }

  public void setNodeColour(String node, int status) {
    try {
      m_process.locateNode(node).setStatus(status);
    }
    catch(NullPointerException e) {
      return;
    }
  }

  public void loadBlankGraph() {
    String file = "";
    String thisLine;
    m_process.clear();
    if (currentGraph.equals("blank")) {
      currentGraph = "top_level.txt";
    }
    file = currentGraph;
    try {
      FileInputStream fin = new FileInputStream("ProcessModels/" + file);
      DataInputStream myInput = new DataInputStream(fin);
      while((thisLine = myInput.readLine()) != null) {
	parseAndLoad(thisLine);
      }
    }
    catch(IOException e) {
      System.err.println("Error: " + e);
    }
    repaint();
  }

  public synchronized void statusUpdate(ProcessStatusUpdate up) {
    Vector changes = up.statusChanges;

    while (statusTables.size() < (up.numberOfCoas + 1)) {
	// +1 because the 0th table is for "Overall".
      statusTables.addElement(new Hashtable());
    }
    
    if (up.containsCompleteState) {
      m_process.clear();
      loadBlankGraph();
      if (changes.isEmpty()) {
	// The model can't easily tell us this because there's no COA.  /\/
	setStatus(0, "develop JFACC guidance", ViewColor.STATUS_POSSIBLE);
      }
    }

    for (Enumeration ce = changes.elements(); ce.hasMoreElements();) {
      StatusChange change = (StatusChange)ce.nextElement();
      if (change.coa >= 1) {
	setStatus(change.coa, change.action, change.status);
      }
      else if (change.coa == StatusChange.COA_OVERALL)
	setStatus(0, change.action, change.status);
      else {
	Debug.assert(change.coa == StatusChange.COA_NONE);
	// Put the status value in every entry on the row.
	for (int coa = 1; coa <= up.numberOfCoas; coa++) {
          setStatus(coa, change.action, change.status);
	}
      }
    }

    repaint();

  }
    
  // need to add further calls here
  public synchronized void paintGraph(int n) {
    m_process.clear();
    switch (n)
      {
      case 0:
	loadBlankGraph();
	setStatus(0, "develop JFACC guidance", 3);
	setStatus(0, "develop targets", 0);
	setStatus(0, "formalize planning advice", 0);
	setStatus(0, "develop COA", 0);
	setStatus(0, "decide to explore alternative COA(s)", 0);
	setStatus(0, "decide to brief on COA(s)", 0);
	setStatus(0, "brief on COA(s)", 0);
	setStatus(0, "create COA", 0);
	setStatus(0, "assign weapons systems", 0);
	setStatus(0, "create support plan", 0);
	setStatus(0, "review COA", 0);
	setStatus(0, "monitor COA-creation process", 0);
	break;
      case 1:
	loadBlankGraph();
	setStatus(0, "develop JFACC guidance", 1);
	setStatus(0, "develop targets", 1);
	setStatus(0, "formalize planning advice", 1);
	setStatus(0, "develop COA", 1);
	setStatus(0, "decide to explore alternative COA(s)", 3);
	setStatus(0, "decide to brief on COA(s)", 3);
	setStatus(0, "brief on COA(s)", 0);
	setStatus(0, "create COA", 1);
	setStatus(0, "assign weapons systems", 1);
	setStatus(0, "create support plan", 1);
	setStatus(0, "review COA", 1);
	setStatus(0, "monitor COA-creation process", 1);
	break;
      case 2:
	loadBlankGraph();
	setStatus(0, "develop JFACC guidance", 3);
	setStatus(0, "develop targets", 0);
	setStatus(0, "formalize planning advice", 0);
	setStatus(0, "develop COA", 0);
	setStatus(0, "decide to explore alternative COA(s)", 1);
	setStatus(0, "decide to brief on COA(s)", 3);
	setStatus(0, "brief on COA(s)", 0);
	setStatus(0, "create COA", 0);
	setStatus(0, "assign weapons systems", 0);
	setStatus(0, "create support plan", 0);
	setStatus(0, "review COA", 0);
	setStatus(0, "monitor COA-creation process", 0);
	break;
      case 3:
	loadBlankGraph();
	setStatus(0, "develop JFACC guidance", 3);
	setStatus(0, "develop targets", 0);
	setStatus(0, "formalize planning advice", 0);
	setStatus(0, "develop COA", 0);
	setStatus(0, "decide to explore alternative COA(s)", 1);
	setStatus(0, "decide to brief on COA(s)", 3);
	setStatus(0, "brief on COA(s)", 0);
	setStatus(0, "create COA", 0);
	setStatus(0, "assign weapons systems", 0);
	setStatus(0, "create support plan", 0);
	setStatus(0, "review COA", 0);
	setStatus(0, "monitor COA-creation process", 0);
	break;
      case 31:
	loadBlankGraph();
	setStatus(0, "develop JFACC guidance", 3);
	setStatus(0, "develop targets", 0);
	setStatus(0, "formalize planning advice", 0);
	setStatus(0, "develop COA", 0);
	setStatus(0, "decide to explore alternative COA(s)", 0);
	setStatus(0, "decide to brief on COA(s)", 3);
	setStatus(0, "brief on COA(s)", 0);
	setStatus(0, "create COA", 0);
	setStatus(0, "assign weapons systems", 0);
	setStatus(0, "create support plan", 0);
	setStatus(0, "review COA", 0);
	setStatus(0, "monitor COA-creation process", 0);
	break;
      case 32:
	loadBlankGraph();
	setStatus(0, "develop JFACC guidance", 3);
	setStatus(0, "develop targets", 0);
	setStatus(0, "formalize planning advice", 0);
	setStatus(0, "develop COA", 0);
	setStatus(0, "decide to explore alternative COA(s)", 1);
	setStatus(0, "decide to brief on COA(s)", 3);
	setStatus(0, "brief on COA(s)", 0);
	setStatus(0, "create COA", 0);
	setStatus(0, "assign weapons systems", 0);
	setStatus(0, "create support plan", 0);
	setStatus(0, "review COA", 0);
	setStatus(0, "monitor COA-creation process", 0);
	break;
      case 33:
	loadBlankGraph();
	setStatus(0, "develop JFACC guidance", 1);
	setStatus(0, "develop targets", 1);
	setStatus(0, "formalize planning advice", 1);
	setStatus(0, "develop COA", 2);
	setStatus(0, "decide to explore alternative COA(s)", 0);
	setStatus(0, "decide to brief on COA(s)", 3);
	setStatus(0, "brief on COA(s)", 0);
	setStatus(0, "create COA", 2);
	setStatus(0, "assign weapons systems", 0);
	setStatus(0, "create support plan", 0);
	setStatus(0, "review COA", 0);
	setStatus(0, "monitor COA-creation process", 3);
	break;
      case 4:
	loadBlankGraph();
	setStatus(0, "develop JFACC guidance", 1);
	setStatus(0, "develop targets", 1);
	setStatus(0, "formalize planning advice", 1);
	setStatus(0, "develop COA", 2);
	setStatus(0, "decide to explore alternative COA(s)", 0);
	setStatus(0, "decide to brief on COA(s)", 3);
	setStatus(0, "brief on COA(s)", 0);
	setStatus(0, "create COA", 2);
	setStatus(0, "assign weapons systems", 0);
	setStatus(0, "create support plan", 0);
	setStatus(0, "review COA", 2);
	setStatus(0, "monitor COA-creation process", 2);
	break;
      case 5:
	loadBlankGraph();
	setStatus(0, "develop JFACC guidance", 1);
	setStatus(0, "develop targets", 1);
	setStatus(0, "formalize planning advice", 1);
	setStatus(0, "develop COA", 2);
	setStatus(0, "decide to explore alternative COA(s)", 0);
	setStatus(0, "decide to brief on COA(s)", 3);
	setStatus(0, "brief on COA(s)", 0);
	setStatus(0, "create COA", 1);
	setStatus(0, "assign weapons systems", 1);
	setStatus(0, "create support plan", 1);
	setStatus(0, "review COA", 2);
	setStatus(0, "monitor COA-creation process", 1);
	break;
      case 6:
	loadBlankGraph();
	setStatus(0, "develop JFACC guidance", 1);
	setStatus(0, "develop targets", 1);
	setStatus(0, "formalize planning advice", 1);
	setStatus(0, "develop COA", 1);
	setStatus(0, "decide to explore alternative COA(s)", 3);
	setStatus(0, "decide to brief on COA(s)", 3);
	setStatus(0, "brief on COA(s)", 0);
	setStatus(0, "create COA", 1);
	setStatus(0, "assign weapons systems", 1);
	setStatus(0, "create support plan", 1);
	setStatus(0, "review COA", 1);
	setStatus(0, "monitor COA-creation process", 1);
	break;
      }
  }

  private void parseAndLoad (String in) {
    switch (in.charAt(0))
      {
      case '%':
	break;
      case 'F':
        StringTokenizer st = new StringTokenizer(in, " ");
	Integer tempX = null;
	Integer tempY = null;
	Integer tempWidth = null;
	Integer tempHeight = null;
	int i = 0;
	for  (Enumeration  e  = st; e.hasMoreElements()  ; i++)  {
	  String temp = (String) e.nextElement();
	  switch (i) 
	    {
	    case 1:
	      tempX = new Integer(temp);
	      break;
	    case 2:
	      tempY = new Integer(temp);
	      break;
	    case 3:
	      tempWidth = new Integer(temp);
	      break;
	    case 4:
	      tempHeight = new Integer(temp);
	      break;
	    }
	}
	break;
      case 'N':
	String tmpKey = null;
	String tmpLabel = null;
	Integer tmpX = null;
	Integer tmpY = null;
	String tmpType = null;
	Integer tmpStatus = null;
	String tmpShadow = null;
	String tmpExpansion = null;
	StringTokenizer st1 = new StringTokenizer(in, "{}");
	int i1 = 0;
	for  (Enumeration  e  = st1; e.hasMoreElements()  ; i1++)  {	
	  String temp = (String) e.nextElement();
	  switch (i1) {
	  case 1:
	    tmpKey = temp;
	    tmpLabel = temp;
	    break;
	  case 2:
	    StringTokenizer st2 = new StringTokenizer(temp, " ");
	    int j = 0;
	    for  (Enumeration  e2  = st2;
		  e2.hasMoreElements()  ; j++)  {	
	      String temp2 = (String) e2.nextElement();
	      switch (j) {
	      case 0:
		tmpX = new Integer(temp2);
		break;
	      case 1:
		tmpY = new Integer(temp2);
		break;
	      case 2:
		tmpType = temp2;
		break;
	      case 3:
		tmpStatus = new Integer(temp2);
		break;
	      case 4:
		tmpShadow = temp2;
		break;
	      }
	    }
	    break;
	  case 3:
	    tmpExpansion = temp;
	    break;
	  }
	}
	Node tmpNode = null;
	if (tmpType.equals("S")) {
	  tmpNode = new SpecialNode(tmpLabel,tmpX.intValue(),
				    tmpY.intValue());
	}
	else {
          tmpNode = new Node(tmpLabel,tmpX.intValue(),
			     tmpY.intValue());
	}
	if (tmpType.equals("D")) {
	  tmpNode.dummyNode = true;
	}
	tmpNode.setStatus(tmpStatus.intValue());
	tmpNode.m_selected = false;
	tmpNode.shadow = ((tmpShadow.equals("Y") ? true : false));
	tmpNode.expansion = tmpExpansion;
	m_process.addNode(tmpNode);
	break;
      case 'E':
	String tmpFrom = null;
	String tmpTo = null;
	StringTokenizer st3 = new StringTokenizer(in, "{}");
	int i3 = 0;
	for  (Enumeration  e  = st3; e.hasMoreElements()  ; i3++)  {	
	  String temp = (String) e.nextElement();
	  switch (i3) 
	    {
	    case 1:
              tmpFrom = temp;
	      break;
	    case 3:
	      tmpTo = temp;
	      break;
	    }
	}
	m_process.addEdge(tmpFrom,tmpTo);
	break;
      }
  }
}
