/* File: LispTest.java
 * Contains: Main program for testing Lisp classes.
 * Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Created: January 1998
 * Updated: Thu Jun 22 14:24:33 2000 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

package ix.test;

import java.util.*;
import ix.util.*;
import ix.util.lisp.*;


/** A class for testing */

class LispTest {

    public static void main(String[] argv) {

	Debug.noteln("An empty list:", Lisp.list());

	Debug.noteln("A list of 3 strings:", Lisp.list("a", "b", "c"));

	Debug.noteln("Length of () =", Lisp.list().length());

	Debug.noteln("Length of (a b) =", Lisp.list("a", "b").length());

	Debug.assert(Symbol.intern("apple") == Symbol.intern("apple"));

	Debug.noteln("A keyword:", Symbol.intern(":apple"));

	{
	    Symbol[] syms = {Symbol.intern("apple"),
			     Symbol.intern(":apple"),
			     Symbol.intern("?apple"),
			     Keyword.intern(":apple")};
	    for (int sy = 0; sy < syms.length; sy++) {
		Debug.noteln("Class of " + syms[sy] + " ==> " +
			     syms[sy].getClass());
	    }
	}

	// What if we try to instantiate List?  The compiler objects
	//   (as it should).
	// Debug.noteln("A List:", new LList());

	Debug.noteln("Enumerations:");
	Debug.noteEnumeration(Lisp.list().elements());
	Debug.noteEnumeration(Lisp.list("a", "b", "c").elements());

	Debug.noteln("elementAt:");
	{
	    LList l = Lisp.list("a", "b", "c");
	    Debug.noteln("0: " + l.elementAt(0));
	    Debug.noteln("1: " + l.elementAt(1));
	    Debug.noteln("2: " + l.elementAt(2));
	    Debug.noteln("3: " + l.elementAt(3));

	    Debug.noteln("1st elt of nil: " + Lisp.list().elementAt(1));
	}

	Debug.noteln("Append (a b c) (d e) ==>",
		     Lisp.list("a", "b", "c").append(Lisp.list("d", "e")));
	Debug.noteln("Append () (d e) ==>",
		     Lisp.list().append(Lisp.list("d", "e")));

	Debug.noteln("Equal (a b c) (a b c) ==> " +
		     Lisp.equal(Lisp.list("a", "b", "c"),
				Lisp.list("a", "b", "c")));
	Debug.noteln("Equal (a b c) (a c b) ==> " +
		     Lisp.equal(Lisp.list("a", "b", "c"),
				Lisp.list("a", "c", "b")));
	Debug.noteln("Equal (a b c) (a b) ==> " +
		     Lisp.equal(Lisp.list("a", "b", "c"),
				Lisp.list("a", "b")));
	Debug.noteln("Equal () () ==> " +
		     Lisp.equal(Lisp.list(), Lisp.list()));


	Debug.noteln("find () a ==> " + Lisp.list().find(Symbol.intern("a")));
	Debug.noteln("find (a b c) a ==> " +
		     stringToList("(a b c)").find(Symbol.intern("a")));
	Debug.noteln("find (a b c) b ==> " +
		     stringToList("(a b c)").find(Symbol.intern("b")));
	Debug.noteln("find (a b c) c ==> " +
		     stringToList("(a b c)").find(Symbol.intern("c")));
	Debug.noteln("find (a b c) d ==> " +
		     stringToList("(a b c)").find(Symbol.intern("d")));

	Debug.noteln("get (a b c) a ==>",
		     stringToList("(a b c)").get(Symbol.intern("a")));
	Debug.noteln("get (a b c) b ==>",
		     stringToList("(a b c)").get(Symbol.intern("b")));
	Debug.noteln("get (a b c) c ==>",
		     stringToList("(a b c)").get(Symbol.intern("c")));


	Debug.noteln("Reverse (a b c) ==>",
		     stringToList("(a b c)").reverse());

	Debug.noteln("Take (a b c) 0 ==>",
		     stringToList("(a b c)").take(0));
	Debug.noteln("Take (a b c) 1 ==>",
		     stringToList("(a b c)").take(1));
	Debug.noteln("Take (a b c) 2 ==>",
		     stringToList("(a b c)").take(2));
	Debug.noteln("Take (a b c) 3 ==>",
		     stringToList("(a b c)").take(3));
	Debug.noteln("Take (a b c) 4 ==>",
		     stringToList("(a b c)").take(4));

	Debug.noteln("Drop (a b c) 0 ==>",
		     stringToList("(a b c)").drop(0));
	Debug.noteln("Drop (a b c) 1 ==>",
		     stringToList("(a b c)").drop(1));
	Debug.noteln("Drop (a b c) 2 ==>",
		     stringToList("(a b c)").drop(2));
	Debug.noteln("Drop (a b c) 3 ==>",
		     stringToList("(a b c)").drop(3));
	Debug.noteln("Drop (a b c) 4 ==>",
		     stringToList("(a b c)").drop(4));

	/* Mapping functions */

	Debug.noteln
	    ("mapcar (lambda (x) (list x x)) (a b c) ==>",
	      stringToList("(a b c)").mapcar(new Function1() {
		  public Object funcall(Object a) {
		      return Lisp.list(a, a);
		  }
	      }));

	Debug.noteln("mapc print (a b c):");
	stringToList("(a b c)").mapc(new Function1() {
	    public Object funcall(Object a) {
		System.out.println(a);
		return null;
	    }
	});

	Debug.noteln
	    ("flatmap (lambda (x) (list x x)) (a b c) ==>",
	      stringToList("(a b c)").flatmap(new Function1() {
		  public Object funcall(Object a) {
		      return Lisp.list(a, a);
		  }
	      }));

	Debug.note("walkTree (a (b c (d)) ((e))) print:");
	stringToList("(a (b c (d)) ((e)))").walkTree(new Function1() {
	    public Object funcall(Object a) {
		System.out.print(" " + a);
		return null;
	    }
	});
	Debug.noteln("");

	/* replaceAll */

	Debug.noteln(
	    "replaceAll a with b in (a q u a) ==>",
	    stringToList("(a q u a)").replaceAll(
	        Symbol.intern("a"),
		Symbol.intern("b")));
	    

	/* ListCollector */

	{
	    ListCollector c = new ListCollector();
	    c.addElement(Symbol.intern("a"));
	    c.concList(stringToList("(b c d)"));
	    c.addElement(Symbol.intern("e"));
	    Debug.noteln("collection a (b c d) e ==>", c.contents());

	    c = new ListCollector();
	    c.pushElement("c");
	    c.addElement("d");
	    c.pushElement("b");
	    c.pushElement("a");
	    c.addElement("e");
	    Debug.noteln("collection made w/ push and add:", c.contents());

	    LList abc = stringToList("(a b c)");
	    c = new ListCollector();
	    c.appendList(abc);
	    c.appendList(abc);
	    Debug.noteln("appendList-ing the same (==) List twice");
	    Debug.noteln("  collection contains", c.contents());
	    Debug.noteln("  original list is now", abc);

	    abc = stringToList("(a b c)"); // make sure
	    c = new ListCollector();
	    abc.mapc(c.elementAdder());
	    Debug.noteln("after abc.mapc(c.elementAdder())", c.contents());
	    abc.mapc(c.elementAdder());
	    Debug.noteln("again! ==>", c.contents());
	    abc.mapc(c.elementPusher());
	    Debug.noteln("after pushing", c.contents());

	}

	/* Cloning and hashing (also presumably tests equals) */

	{
	    LList lis = stringToList("(this and that)");
	    LList lclone = (LList)lis.clone();
	    LList lalt = stringToList("(this and that)");
	    Hashtable ht = new Hashtable();

	    Debug.noteln("List: " + lis + "; clone: " + lclone);
	    Debug.noteln("Clone differs?", (lis != lclone) ? "yes" : "no");
	    ht.put(lis, "yes");
	    Debug.noteln("Clone found in Hashtable?", ht.get(lclone));
	    Debug.noteln("Alt found in Hashtable?", ht.get(lalt));

	}

	/* Intersection */
	Debug.noteln("intersect (a b c d) (a c) ==>",
	   stringToList("(a b c d)").intersect(stringToList("(a c)")));

	/* Permute */
	{
	    LList example = stringToList("(a b c)");
	    Debug.noteln("Permuting", example);
	    for (int i = 0; i < 5; i++) {
		Debug.noteln("--", example.permute());
	    }
	}

    }

    // Cons.clone(), like Object.clone() is a protected method,  [not any more]
    // so we (heh, heh) have to get around that.
    /*
    static class CopyableCons extends Cons {
	public CopyableCons(Object a, LList b) {
	    super(a, b);
	}
	public Object shallowCopyTrick() {
	    return this.clone();
	}
    }
    */

    static LList stringToList(String s) {
	return (LList)Lisp.readFromString(s);
    }

}
