/* File: MatchTest.java
 * Contains: Main program for testing MatchTables
 * Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Created: January 1998
 * Updated: Thu Apr  2 02:46:08 1998 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

package ix.test;

import java.util.*;
import ix.util.*;
import ix.util.match.*;


/** A class for testing MatchTable pattern-matching. */

class MatchTest {

    public static void main(String[] argv) {

	StringMatchTable table = new StringMatchTable();

	table.addPattern("apple");
	table.addPattern("pie");

	/* Now loop, matching against what the user types. */

	CommandLoop loop = new CommandLoop(table);
	loop.setPrompt("How about:");
	loop.run();

    }

}


/* StringMatchTable's addPattern automatically adds an appropriate
   MatchCase, given a String pattern. */

class StringMatchTable extends MatchTable {

    void addPattern(String pattern) {
	addCase(new StringMatchCase(pattern));
    }

}


/* StringMatchCase provides incredibly simple matching using the
   String equals method. */

class StringMatchCase extends MatchCase {

    StringMatchCase(String pattern) {
	this.pattern = pattern;
    }

    public Object tryMatch(Object data) {
	if (pattern.equals(data))
	    return data;
	else
	    return null;
    }

    public Object ifSelected(Object data, Object matchResult) {
	return "Cool: " + matchResult;
    }

}
