/*** File: Process.java
*** Contains: Class defining the main process element for CPE.
*** Author: Steve Polyak <Steve_Polyak@ed.ac.uk>
*** Created: June 1997
*** Updated: Fri Jun 16 14:25:05 2000 by Jeff Dalton
*/

package ix.view.graph;

import java.util.*;
import java.awt.*;
import ix.util.*;

class CPEProcess {
  // Link back to the ViewGraph panel
  ViewGraph m_panel = null;
  
  // Elements kept as a vector of nodes, edges
  Vector m_nodes;
  Vector m_edges;
  
  // Graphics elements
  final Color m_edgeColor = Color.black;
  Image m_offscreen;
  Dimension m_offscreensize;
  Dimension m_dimension;
  Graphics m_offgraphics;

  // A record of the most recently hit node
  Node m_hitNode = null;
  
  // Constructor saves a reference back to the panel
  CPEProcess(ViewGraph panel) {
    m_panel = panel;
    m_nodes = new Vector();
    m_edges = new Vector();		
  }
  
  public void clear() {
    m_edges.removeAllElements();
    m_nodes.removeAllElements();  
  }
  
  public void newProcess() {
    m_dimension = m_panel.size();
    clear();
  }
  
  void addNode(Node node) {
    m_nodes.addElement(node);
  }
  
  Node locateNode(String key) {
    Node found = null;
    Node temp;

    // For now, a simple linear search
    for (Enumeration e = m_nodes.elements(); e.hasMoreElements();) {
      temp = (Node) e.nextElement();
      if (temp.m_key.equals(key)) {
        found = temp;
        break; 
      }
    }	
    return found;
  }

  void addEdge(String from, String to) {
    Node nodeFrom, nodeTo;
    nodeFrom = locateNode(from);
    nodeTo = locateNode(to);
    
    m_edges.addElement(new Edge(nodeFrom, nodeTo));
  }
  
  void update(Graphics g) {
    
    Dimension d = m_panel.size();    
    
    if ((m_offscreen == null) || 
	(d.width != m_offscreensize.width) ||
	(d.height != m_offscreensize.height)) {
      m_offscreen = m_panel.createImage(d.width, d.height);
      m_offscreensize = d;
      m_offgraphics = m_offscreen.getGraphics();
      m_offgraphics.setFont(m_panel.getFont());
    }
    
    m_offgraphics.setColor(m_panel.getBackground());
    m_offgraphics.fillRect(0, 0, d.width, d.height);
    FontMetrics fm = m_offgraphics.getFontMetrics();
    
    // Draw each edge
    for (Enumeration e = m_edges.elements(); e.hasMoreElements();) {	
      ((Edge) e.nextElement()).draw(m_offgraphics, fm);
    }	
    
    // Draw each node
    for (Enumeration e = m_nodes.elements(); e.hasMoreElements();) {	
      ((Node) e.nextElement()).draw(m_offgraphics, fm);
    }
    
    // Draw each edge arrow
    for (Enumeration e = m_edges.elements(); e.hasMoreElements();) {
      ((Edge) e.nextElement()).drawArrow(m_offgraphics, fm);
    }	
    
    g.drawImage(m_offscreen, 0, 0, null);
  }

  public boolean mouseUp(Event evt, int x, int y) {
    Node temp;
    m_hitNode = null;

    for (int intSize = m_nodes.size(); intSize > 0; intSize--) {
      temp = (Node) m_nodes.elementAt(intSize - 1);
      if (temp.m_rect.inside(x,y)) {
	m_hitNode = temp;
	if (m_hitNode.expansion != null) {
	  m_panel.setGraph(m_hitNode.expansion);
	}
	break;
      }
    }
    return true;
  }
}
